@@ -1,7 +1,7 @@
 package Complete::Util;
 
-our $DATE = '2015-09-18'; # DATE
-our $VERSION = '0.38'; # VERSION
+our $DATE = '2015-11-28'; # DATE
+our $VERSION = '0.39'; # VERSION
 
 use 5.010001;
 use strict;
@@ -145,12 +145,31 @@ Will sort the resulting completion list, so you don't have to presort the array.
 
 _
     args => {
-        word     => { schema=>[str=>{default=>''}], pos=>0, req=>1 },
-        array    => { schema=>['array*'=>{of=>'str*'}], req=>1 },
-        ci       => { schema=>['bool'] },
-        exclude  => { schema=>['array*'] },
-        fuzzy    => { schema=>['int*', min=>0] },
-        map_case => {
+        word      => { schema=>[str=>{default=>''}], pos=>0, req=>1 },
+        array     => { schema=>['array*'=>{of=>'str*'}], req=>1 },
+        ci        => {
+            schema=>['bool'],
+        },
+        exclude   => { schema=>['array*'] },
+        word_mode => {
+            summary => 'Whether to enable word-mode matching',
+            schema=>['bool'],
+            description => <<'_',
+
+See `Complete::Setting` for more detail on word-mode matching.
+
+_
+        },
+        fuzzy     => {
+            summary => 'Set fuzziness for fuzzy matching',
+            schema=>['int*', min=>0],
+            description => <<'_',
+
+See `Complete::Setting` for more detail on fuzzy matching.
+
+_
+        },
+        map_case  => {
             summary => 'Treat _ (underscore) and - (dash) as the same',
             schema  => ['bool'],
         },
@@ -161,21 +180,16 @@ _
     },
 };
 sub complete_array_elem {
-    state $code_editdist = do {
-        if (eval { require Text::Levenshtein::XS; 1 }) {
-            \&Text::Levenshtein::XS::distance;
-        } else {
-            \&__editdist;
-        }
-    };
+    state $code_editdist;
 
     my %args  = @_;
 
-    my $array    = $args{array} or die "Please specify array";
-    my $word     = $args{word} // "";
-    my $ci       = $args{ci} // $Complete::Setting::OPT_CI;
-    my $fuzzy    = $args{fuzzy} // $Complete::Setting::OPT_FUZZY;
-    my $map_case = $args{map_case} // $Complete::Setting::OPT_MAP_CASE;
+    my $array     = $args{array} or die "Please specify array";
+    my $word      = $args{word} // "";
+    my $ci        = $args{ci} // $Complete::Setting::OPT_CI;
+    my $word_mode = $args{word_mode} // $Complete::Setting::OPT_WORD_MODE;
+    my $fuzzy     = $args{fuzzy} // $Complete::Setting::OPT_FUZZY;
+    my $map_case  = $args{map_case} // $Complete::Setting::OPT_MAP_CASE;
 
     return [] unless @$array;
 
@@ -183,21 +197,50 @@ sub complete_array_elem {
     my $wordn = $ci ? uc($word) : $word; $wordn =~ s/_/-/g if $map_case;
 
     my @words;
+    my @arrayn;
     for my $el (@$array) {
         my $eln = $ci ? uc($el) : $el; $eln =~ s/_/-/g if $map_case;
+        push @arrayn, $eln; # so we don't have to calculate again
         next unless 0==index($eln, $wordn);
         push @words, $el;
     }
 
+    {
+        last unless $word_mode && !@words;
+        my @split_wordn = $wordn =~ /(\w+)/g;
+        last unless @split_wordn > 1;
+        my $re = '\A';
+        for my $i (0..$#split_wordn) {
+            $re .= '(?:\W+\w+)*\W+' if $i;
+            $re .= quotemeta($split_wordn[$i]).'\w*';
+        }
+        #say "D:re=$re";
+        $re = qr/$re/;
+
+        for my $i (0..$#{$array}) {
+            next unless $arrayn[$i] =~ $re;
+            push @words, $array->[$i];
+        }
+    }
+
     if ($fuzzy && !@words) {
+        $code_editdist //= do {
+            if (eval { require Text::Levenshtein::XS; 1 }) {
+                \&Text::Levenshtein::XS::distance;
+            } else {
+                \&__editdist;
+            }
+        };
+
         my $factor = 1.3;
         my $x = -1;
         my $y = 1;
 
         my %editdists;
       ELEM:
-        for my $el (@$array) {
-            my $eln = $ci ? uc($el) : $el; $eln =~ s/_/-/g if $map_case;
+        for my $i (0..$#{$array}) {
+            my $eln = $arrayn[$i];
+
             for my $l (length($wordn)-$y .. length($wordn)+$y) {
                 next if $l <= 0;
                 my $chopped = substr($eln, 0, $l);
@@ -214,7 +257,7 @@ sub complete_array_elem {
                 );
                 #say "D: d(".($ci ? $wordu:$word).",$chopped)=$d (maxd=$maxd)";
                 next unless $d <= $maxd;
-                push @words, $el;
+                push @words, $array->[$i];
                 next ELEM;
             }
         }
@@ -235,11 +278,12 @@ $SPEC{complete_hash_key} = {
     v => 1.1,
     summary => 'Complete from hash keys',
     args => {
-        word     => { schema=>[str=>{default=>''}], pos=>0, req=>1 },
-        hash     => { schema=>['hash*'=>{}], req=>1 },
-        ci       => { schema=>['bool'] },
-        fuzzy    => { schema=>['int*', min=>0] },
-        map_case => { schema=>['bool'] },
+        word      => { schema=>[str=>{default=>''}], pos=>0, req=>1 },
+        hash      => { schema=>['hash*'=>{}], req=>1 },
+        ci        => { schema=>['bool'] },
+        word_mode => { schema=>['bool'] },
+        fuzzy     => { schema=>['int*', min=>0] },
+        map_case  => { schema=>['bool'] },
     },
     result_naked => 1,
     result => {
@@ -248,17 +292,18 @@ $SPEC{complete_hash_key} = {
 };
 sub complete_hash_key {
     my %args  = @_;
-    my $hash     = $args{hash} or die "Please specify hash";
-    my $word     = $args{word} // "";
-    my $ci       = $args{ci} // $Complete::Setting::OPT_CI;
-    my $fuzzy    = $args{fuzzy} // $Complete::Setting::OPT_FUZZY;
-    my $map_case = $args{map_case} // $Complete::Setting::OPT_MAP_CASE;
+    my $hash      = $args{hash} or die "Please specify hash";
+    my $word      = $args{word} // "";
+    my $ci        = $args{ci} // $Complete::Setting::OPT_CI;
+    my $word_mode = $args{word_mode} // $Complete::Setting::OPT_WORD_MODE;
+    my $fuzzy     = $args{fuzzy} // $Complete::Setting::OPT_FUZZY;
+    my $map_case  = $args{map_case} // $Complete::Setting::OPT_MAP_CASE;
 
     my @array = $ci ?
         (sort {uc($a) cmp uc($b)} keys %$hash) : (sort keys %$hash);
     complete_array_elem(
         word=>$word, array=>\@array,
-        ci=>$ci, fuzzy=>$fuzzy, map_case=>$map_case,
+        ci=>$ci, word_mode=>$word_mode, fuzzy=>$fuzzy, map_case=>$map_case,
     );
 }
 
@@ -386,7 +431,7 @@ Complete::Util - General completion routine
 
 =head1 VERSION
 
-This document describes version 0.38 of Complete::Util (from Perl distribution Complete-Util), released on 2015-09-18.
+This document describes version 0.39 of Complete::Util (from Perl distribution Complete-Util), released on 2015-11-28.
 
 =head1 DESCRIPTION
 
@@ -400,6 +445,8 @@ Make sure we return completion answer in array form.
 This is the reverse of C<hashify_answer>. It accepts a hash or an array. If it
 receives a hash, will return its C<words> key.
 
+This function is not exported by default, but exportable.
+
 Arguments ('*' denotes required arguments):
 
 =over 4
@@ -428,6 +475,8 @@ a directory, or a module name. You can do something like this:
 If a completion answer has a metadata C<final> set to true, then that answer is
 used as the final answer without any combining with the other answers.
 
+This function is not exported by default, but exportable.
+
 Arguments ('*' denotes required arguments):
 
 =over 4
@@ -450,6 +499,8 @@ Complete from array.
 
 Will sort the resulting completion list, so you don't have to presort the array.
 
+This function is not exported by default, but exportable.
+
 Arguments ('*' denotes required arguments):
 
 =over 4
@@ -462,12 +513,22 @@ Arguments ('*' denotes required arguments):
 
 =item * B<fuzzy> => I<int>
 
+Set fuzziness for fuzzy matching.
+
+See C<Complete::Setting> for more detail on fuzzy matching.
+
 =item * B<map_case> => I<bool>
 
 Treat _ (underscore) and - (dash) as the same.
 
 =item * B<word>* => I<str> (default: "")
 
+=item * B<word_mode> => I<bool>
+
+Whether to enable word-mode matching.
+
+See C<Complete::Setting> for more detail on word-mode matching.
+
 =back
 
 Return value:  (array)
@@ -477,6 +538,8 @@ Return value:  (array)
 
 Complete from hash keys.
 
+This function is not exported by default, but exportable.
+
 Arguments ('*' denotes required arguments):
 
 =over 4
@@ -491,6 +554,8 @@ Arguments ('*' denotes required arguments):
 
 =item * B<word>* => I<str> (default: "")
 
+=item * B<word_mode> => I<bool>
+
 =back
 
 Return value:  (array)
@@ -506,6 +571,8 @@ hash form.
 
 Then will add keys from C<meta> to the hash.
 
+This function is not exported by default, but exportable.
+
 Arguments ('*' denotes required arguments):
 
 =over 4
@@ -535,7 +602,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/Complete-U
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/perlancar/perl-Complete-Util>.
+Source repository is at L<https://github.com/sharyanto/perl-SHARYANTO-Complete-Util>.
 
 =head1 BUGS
 
